unit MWconnU;

// Please, don't delete this comment. \\
(*
  Copyright Owner: coltishWARE (Kevin Niehage)
  Copyright Year : 2007-2008

  Unit   : MWconnU (platform dependant)
  Version: 0.6c

  Contact E-Mail            : development@coltishWARE.com
  Contact Mobile            : +49-(0)-177-69-4321-8
  Contact Telephone Berlin  : +49-(0)-30-69-20-96-78
  Contact Telephone Cottbus : +49-(0)-355-28925-88-17-55
  Contact Telephone Mannheim: +49-(0)-621-483-4664
*)
// Please, don't delete this comment. \\

(*
  Description:

  This unit contains the shared memory interface
  implementation for the application MWconn by
  Markus B. Weber.

  It is a plain C to Pascal translation.
*)

(*
  Change Log:

  [Version 0.6c] (25.02.2008: structure version 3 release)
  - compiler switches updated
  - data structure updated

  [Version 0.5c] (12.02.2008: 4.0 release)
  - compiler switches updated
  - constant MWconnFileGlobal added that specifies the global shared memory identifier
  - OpenMWconnIO() checks what shared memory access is granted (none|read|write|all)
  - OpenMWconnIO() checks whether shared memory is accessable globally

  [Version 0.4.1c] (10.01.2008: 3.11 release)
  - compiler switches updated

  [Version 0.4c] (21.12.2007: versioning release)
  - moved compiler switching to seperate include file (MWconnU.inc)
  - structures are now references by their version (and not the version of MWconn)
  - moved structure versions to seperate include files (MWconnU[N].inc)

  [Version 0.3c] (11.12.2007: 3.10 release)
  - renamed types and compiler switches

  [Version 0.3b] (16.11.2007: 3.9C release)
  - support for 3.9C (not just 3.9 stable)

  [Version 0.2c] (13.11.2007: extended release)
  - calling method split into three methods (shared memory accessible now)
  - command constants introduced

  [Version 0.1c] (13.11.2007: initial release)
  - structure definition translated from C to Pascal
  - calling method written
*)

interface

uses
  Windows,
  SysUtils;

// unit-specific constants
const
  cCopyrightOwner = 'coltishWARE (Kevin Niehage)';
  cCopyrightYear  = '2007-2008';
  cName           = 'MWconnU';
  cReleaseDate    = '25.02.2008';
  cReleaseName    = 'structure version 3 release';
  cVersion        = '0.6c';

// the different Structure-Versions are included here
{$I MWconnU1.inc}
{$I MWconnU2.inc}
{$I MWconnU3.inc}

// all compiler switches are included here
// additionally the type TMWconnIO is defined
// depending on which version to use
{$I MWconnU.inc}

// has to be located here in order to work with the
// versioning through *.inc files
type
  PMWconnIO = ^TMWconnIO;

// defines the access levels there are available for shared memory  
type
  TAccessMode = (amNone, amRead, amWrite, amAll);

// MWconn-specific constants
const
  MWconnFile       = 'MWCONN_IO';
  MWconnFileGlobal = 'Global\' + MWconnFile;

  MWconn_Connect    = 'MWCONN_CONNECT';
  MWconn_Disconnect = 'MWCONN_DISCONNECT';
  MWconn_Reconnect  = 'MWCONN_RECONNECT';
  MWconn_End        = 'MWCONN_END';

// opens the shared memory interface of MWconn
// use CloseMWconnIO() to close it again
function OpenMWconnIO(var aFile : THandle; var aMapping : PMWconnIO) : TAccessMode;

// closes an already open shared memory interface
// of MWconn opened with OpenMWconnIO()
procedure CloseMWconnIO(var aFile : THandle; var aMapping : PMWconnIO);

// returns a data snapshot returned by MWconn
function ReturnMWconnIO(var aData : TMWconnIO) : Boolean;

// duration values less than 0 are replaced by empty strings
// durations are millisecond values
// the result is: ":<aWaitDuration>:<aReadDuration>:<aCommand>"
function BuildATCommand(const aCommand : String; const aWaitDuration : LongInt; const aReadDuration : LongInt) : String;

implementation

// opens the shared memory interface of MWconn
// use CloseMWconnIO() to close it again
function OpenMWconnIO(var aFile : THandle; var aMapping : PMWconnIO) : TAccessMode;
  function MapMWconnIO(const aMappedFile : String; const aMapMode : Cardinal; var aFile : THandle; var aMapping : PMWconnIO) : Boolean;
  begin
    Result := false;

    aFile := OpenFileMapping(aMapMode, false, PChar(AMappedFile));
    if (aFile <> 0) then
    begin
      try
        aMapping := MapViewOfFile(aFile, aMapMode, 0, 0, SizeOf(TMWconnIO));

        Result := (aMapping <> nil);
      except
        CloseHandle(aFile);
      end;
    end;
  end;
type
  TMapModeData = record
    AccessMode : TAccessMode;
    MapMode    : Cardinal;
  end;
const
  cMapFile : array [0..1] of String       = (MWconnFileGlobal, MWconnFile);
  cMapMode : array [0..2] of TMapModeData = ((AccessMode : amAll;   MapMode : FILE_MAP_ALL_ACCESS;),
                                             (AccessMode : amWrite; MapMode : FILE_MAP_WRITE;),
                                             (AccessMode : amRead;  MapMode : FILE_MAP_READ;));
var
  lIndexA : Byte;
  lIndexB : Byte;
begin
  Result := amNone;

  for lIndexA := Low(cMapFile) to High(cMapFile) do
  begin
    for lIndexB := Low(cMapMode) to High(cMapMode) do
    begin
      if MapMWconnIO(cMapFile[lIndexA], cMapMode[lIndexB].MapMode, aFile, aMapping) then
        Result := cMapMode[lIndexB].AccessMode;

      if (Result <> amNone) then
        Break;
    end;

    if (Result <> amNone) then
      Break;
  end;
end;

// closes an already open shared memory interface
// of MWconn opened with OpenMWconnIO()
procedure CloseMWconnIO(var aFile : THandle; var aMapping : PMWconnIO);
begin
  if (aMapping <> nil) then
    UnmapViewOfFile(aMapping);
  if (aFile <> 0) then
    CloseHandle(aFile);

  aFile    := 0;
  aMapping := nil;
end;

// returns a data snapshot returned by MWconn
function ReturnMWconnIO(var aData : TMWconnIO) : Boolean;
var
  lBuffer : PMWconnIO;
  lHandle : THandle;
begin
  Result := false;

  if (OpenMWconnIO(lHandle, lBuffer) <> amNone) then
  begin
    try
      CopyMemory(@aData, lBuffer, SizeOf(TMWconnIO));

      Result := true;
    finally
      CloseMWconnIO(lHandle, lBuffer);
    end;
  end;
end;

// duration values less than 0 are replaced by empty strings
// durations are millisecond values
// the result is: ":<aWaitDuration>:<aReadDuration>:<aCommand>"
function BuildATCommand(const aCommand : String; const aWaitDuration : LongInt; const aReadDuration : LongInt) : String;
const
  cSeperator = ':';
begin
  Result := cSeperator;

  if (aWaitDuration >= 0) then
    Result := Result + IntToStr(aWaitDuration);
  Result := Result + cSeperator;

  if (aReadDuration >= 0) then
    Result := Result + IntToStr(aReadDuration);
  Result := Result + cSeperator;

  Result := Result + aCommand;
end;

end.
